@@ -1,4 +1,4 @@
-// Copyright 2011-2024 David Robillard <d@drobilla.net>
+// Copyright 2011-2025 David Robillard <d@drobilla.net>
 // SPDX-License-Identifier: ISC
 
 #include <zix/btree.h>
@@ -14,9 +14,9 @@
 // #define ZIX_BTREE_SORTED_CHECK 1
 
 // Define ZixShort as an integer type half the size of a pointer
-#if UINTPTR_MAX >= UINT32_MAX
+#if UINTPTR_MAX > UINT32_MAX // 64-bit
 typedef uint32_t ZixShort;
-#else
+#else // 32-bit
 typedef uint16_t ZixShort;
 #endif
 
@@ -102,23 +102,15 @@ zix_btree_new(ZixAllocator* const       allocator,
 
   assert(cmp);
 
-  ZixBTree* const t = (ZixBTree*)zix_aligned_alloc(
-    allocator, ZIX_BTREE_PAGE_SIZE, ZIX_BTREE_PAGE_SIZE);
-
-  if (!t) {
-    return NULL;
+  ZixBTree* const t = (ZixBTree*)zix_malloc(allocator, sizeof(ZixBTree));
+  if (t) {
+    t->allocator = allocator;
+    t->root      = NULL;
+    t->cmp       = cmp;
+    t->cmp_data  = cmp_data;
+    t->size      = 0U;
   }
 
-  if (!(t->root = zix_btree_node_new(allocator, true))) {
-    zix_aligned_free(allocator, t);
-    return NULL;
-  }
-
-  t->allocator = allocator;
-  t->cmp       = cmp;
-  t->cmp_data  = cmp_data;
-  t->size      = 0U;
-
   return t;
 }
 
@@ -157,23 +149,25 @@ zix_btree_free(ZixBTree* const           t,
   if (t) {
     zix_btree_clear(t, destroy, destroy_user_data);
     zix_aligned_free(t->allocator, t->root);
-    zix_aligned_free(t->allocator, t);
+    zix_free(t->allocator, t);
   }
 }
 
 void
-zix_btree_clear(ZixBTree* const     t,
-                ZixBTreeDestroyFunc destroy,
-                const void*         destroy_user_data)
+zix_btree_clear(ZixBTree* const           t,
+                const ZixBTreeDestroyFunc destroy,
+                const void* const         destroy_user_data)
 {
-  zix_btree_free_children(t, t->root, destroy, destroy_user_data);
+  if (t->root) {
+    zix_btree_free_children(t, t->root, destroy, destroy_user_data);
+    zix_aligned_free(t->allocator, t->root);
+    t->root = NULL;
+  }
 
-  memset(t->root, 0U, sizeof(ZixBTreeNode));
-  t->root->is_leaf = true;
-  t->size          = 0U;
+  t->size = 0U;
 }
 
-size_t
+ZIX_REALTIME size_t
 zix_btree_size(const ZixBTree* const t)
 {
   assert(t);
@@ -224,7 +218,7 @@ zix_btree_split_child(ZixAllocator* const allocator,
   assert(i < n->n_vals + 1U);
   assert(zix_btree_child(n, i) == lhs);
 
-  const ZixShort max_n_vals = zix_btree_max_vals(lhs);
+  const unsigned max_n_vals = zix_btree_max_vals(lhs);
   ZixBTreeNode*  rhs        = zix_btree_node_new(allocator, lhs->is_leaf);
   if (!rhs) {
     return NULL;
@@ -436,8 +430,13 @@ zix_btree_insert(ZixBTree* const t, void* const e)
 
   ZixStatus st = ZIX_STATUS_SUCCESS;
 
-  // Grow up if necessary to ensure the root is not full
-  if (zix_btree_is_full(t->root)) {
+  if (!t->root) {
+    // Empty tree, create a new leaf root
+    if (!(t->root = zix_btree_node_new(t->allocator, true))) {
+      return ZIX_STATUS_NO_MEM;
+    }
+  } else if (zix_btree_is_full(t->root)) {
+    // Grow up if necessary to ensure the root is not full
     if ((st = zix_btree_grow_up(t))) {
       return st;
     }
@@ -493,8 +492,9 @@ zix_btree_insert(ZixBTree* const t, void* const e)
 static void
 zix_btree_iter_set_frame(ZixBTreeIter* const ti,
                          ZixBTreeNode* const n,
-                         const ZixShort      i)
+                         const unsigned      i)
 {
+  assert(i <= UINT16_MAX);
   ti->nodes[ti->level]   = n;
   ti->indexes[ti->level] = (uint16_t)i;
 }
@@ -740,6 +740,10 @@ zix_btree_remove(ZixBTree* const     t,
   assert(t);
   assert(out);
 
+  if (!t->root) {
+    return ZIX_STATUS_NOT_FOUND;
+  }
+
   ZixBTreeNode* n  = t->root;
   ZixBTreeIter* ti = next;
   ZixStatus     st = ZIX_STATUS_SUCCESS;
@@ -829,6 +833,9 @@ zix_btree_find(const ZixBTree* const t,
   ZixBTreeNode* n = t->root;
 
   *ti = zix_btree_end_iter;
+  if (!n) {
+    return ZIX_STATUS_NOT_FOUND;
+  }
 
   while (!n->is_leaf) {
     bool           equal = false;
@@ -870,6 +877,9 @@ zix_btree_lower_bound(const ZixBTree* const     t,
   ZixBTreeNode* n           = t->root; // Current node
   uint16_t      found_level = 0U;      // Lowest level a match was found at
   bool          found       = false;   // True if a match was ever found
+  if (!n) {
+    return ZIX_STATUS_SUCCESS;
+  }
 
   // Search down until we reach a leaf
   while (!n->is_leaf) {
@@ -919,7 +929,7 @@ zix_btree_lower_bound(const ZixBTree* const     t,
   return ZIX_STATUS_SUCCESS;
 }
 
-void*
+ZIX_REALTIME void*
 zix_btree_get(const ZixBTreeIter ti)
 {
   const ZixBTreeNode* const node  = ti.nodes[ti.level];
@@ -932,7 +942,7 @@ zix_btree_get(const ZixBTreeIter ti)
                        : node->data.inode.vals[index];
 }
 
-ZixBTreeIter
+ZIX_NONBLOCKING ZixBTreeIter
 zix_btree_begin(const ZixBTree* const t)
 {
   assert(t);
@@ -952,7 +962,7 @@ zix_btree_begin(const ZixBTree* const t)
   return iter;
 }
 
-ZixBTreeIter
+ZIX_REALTIME ZixBTreeIter
 zix_btree_end(const ZixBTree* const t)
 {
   (void)t;
@@ -960,7 +970,7 @@ zix_btree_end(const ZixBTree* const t)
   return zix_btree_end_iter;
 }
 
-bool
+ZIX_REALTIME bool
 zix_btree_iter_equals(const ZixBTreeIter lhs, const ZixBTreeIter rhs)
 {
   const size_t indexes_size = ((size_t)lhs.level + 1U) * sizeof(uint16_t);
@@ -969,7 +979,7 @@ zix_btree_iter_equals(const ZixBTreeIter lhs, const ZixBTreeIter rhs)
          (!lhs.nodes[0U] || !memcmp(lhs.indexes, rhs.indexes, indexes_size));
 }
 
-ZixStatus
+ZIX_NONBLOCKING ZixStatus
 zix_btree_iter_increment(ZixBTreeIter* const i)
 {
   assert(i);
@@ -1007,7 +1017,7 @@ zix_btree_iter_increment(ZixBTreeIter* const i)
   return ZIX_STATUS_SUCCESS;
 }
 
-ZixBTreeIter
+ZIX_NONBLOCKING ZixBTreeIter
 zix_btree_iter_next(const ZixBTreeIter iter)
 {
   ZixBTreeIter next = iter;
