@@ -1,9 +1,12 @@
-// Copyright 2011-2021 David Robillard <d@drobilla.net>
+// Copyright 2011-2025 David Robillard <d@drobilla.net>
 // SPDX-License-Identifier: ISC
 
 #include <zix/hash.h>
 
+#include "qualifiers.h"
+
 #include <zix/allocator.h>
+#include <zix/attributes.h>
 #include <zix/status.h>
 
 #include <assert.h>
@@ -25,8 +28,8 @@ struct ZixHashImpl {
   ZixHashEntry*   entries;    ///< Pointer to dynamically allocated table
 };
 
-static const size_t min_n_entries = 4U;
-static const size_t tombstone     = 0xDEADU;
+static ZIX_CONSTEXPR size_t min_n_entries = 4U;
+static ZIX_CONSTEXPR size_t tombstone     = 0xDEADU;
 
 ZixHash*
 zix_hash_new(ZixAllocator* const   allocator,
@@ -71,21 +74,21 @@ zix_hash_free(ZixHash* const hash)
   }
 }
 
-ZixHashIter
+ZIX_NONBLOCKING ZixHashIter
 zix_hash_begin(const ZixHash* const hash)
 {
   assert(hash);
   return hash->entries[0U].value ? 0U : zix_hash_next(hash, 0U);
 }
 
-ZixHashIter
+ZIX_REALTIME ZixHashIter
 zix_hash_end(const ZixHash* const hash)
 {
   assert(hash);
   return hash->n_entries;
 }
 
-ZixHashRecord*
+ZIX_REALTIME ZixHashRecord*
 zix_hash_get(const ZixHash* hash, const ZixHashIter i)
 {
   assert(hash);
@@ -94,7 +97,7 @@ zix_hash_get(const ZixHash* hash, const ZixHashIter i)
   return hash->entries[i].value;
 }
 
-ZixHashIter
+ZIX_NONBLOCKING ZixHashIter
 zix_hash_next(const ZixHash* const hash, ZixHashIter i)
 {
   assert(hash);
@@ -105,7 +108,7 @@ zix_hash_next(const ZixHash* const hash, ZixHashIter i)
   return i;
 }
 
-size_t
+ZIX_REALTIME size_t
 zix_hash_size(const ZixHash* const hash)
 {
   assert(hash);
@@ -303,7 +306,7 @@ zix_hash_plan_insert(const ZixHash* const hash, const ZixHashKey* const key)
     hash, hash->hash_func(key), hash->equal_func, key);
 }
 
-ZixHashRecord*
+ZIX_REALTIME ZixHashRecord*
 zix_hash_record_at(const ZixHash* const hash, const ZixHashInsertPlan position)
 {
   assert(hash);
@@ -330,7 +333,7 @@ zix_hash_insert_at(ZixHash* const          hash,
   entry->value = record;
 
   // Update size and rehash if we exceeded the maximum load
-  const size_t max_load  = hash->n_entries / 2U + hash->n_entries / 8U;
+  const size_t max_load  = (hash->n_entries / 2U) + (hash->n_entries / 8U);
   const size_t new_count = hash->count + 1U;
   if (new_count >= max_load) {
     const ZixStatus st = grow(hash);
