@@ -1,9 +1,10 @@
-// Copyright 2007-2022 David Robillard <d@drobilla.net>
+// Copyright 2007-2025 David Robillard <d@drobilla.net>
 // SPDX-License-Identifier: ISC
 
 #include <zix/filesystem.h>
 
 #include "../errno_status.h"
+#include "../qualifiers.h"
 #include "../system.h"
 #include "../zix_config.h"
 
@@ -26,36 +27,23 @@
 
 #include <dirent.h>
 #include <fcntl.h>
+#include <stdint.h>
 #include <sys/stat.h>
 #include <sys/types.h>
 #include <unistd.h>
 
 #include <errno.h>
 #include <stdbool.h>
-#include <stdint.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
 
-#ifndef MAX
-#  define MAX(a, b) (((a) > (b)) ? (a) : (b))
-#endif
-
 static inline ZixStatus
 zix_posix_status(const int rc)
 {
   return rc ? zix_errno_status(errno) : ZIX_STATUS_SUCCESS;
 }
 
-static uint32_t
-zix_get_block_size(const struct stat* const s1, const struct stat* const s2)
-{
-  const blksize_t b1 = s1->st_blksize;
-  const blksize_t b2 = s2->st_blksize;
-
-  return (b1 > 0 && b2 > 0) ? (uint32_t)MAX(b1, b2) : 4096U;
-}
-
 static ZixStatus
 finish_copy(const int dst_fd, const int src_fd, const ZixStatus status)
 {
@@ -158,14 +146,16 @@ copy_blocks(const int    src_fd,
             void* const  block,
             const size_t block_size)
 {
-  ssize_t n_read = 0;
-  while ((n_read = read(src_fd, block, block_size)) > 0) {
+  ZixStatus st     = ZIX_STATUS_SUCCESS;
+  ssize_t   n_read = 0;
+
+  while (!st && (n_read = read(src_fd, block, block_size)) > 0) {
     if (write(dst_fd, block, (size_t)n_read) != n_read) {
-      return zix_errno_status(errno);
+      st = zix_errno_status(errno);
     }
   }
 
-  return ZIX_STATUS_SUCCESS;
+  return st;
 }
 
 ZixStatus
@@ -175,7 +165,8 @@ zix_copy_file(ZixAllocator* const  allocator,
               const ZixCopyOptions options)
 {
   ZixStatus st = ZIX_STATUS_SUCCESS;
-  (void)st;
+
+  errno = 0;
 
 #if USE_CLONEFILE
   // Try to copy via the kernel on MacOS to take advantage of CoW
@@ -186,7 +177,7 @@ zix_copy_file(ZixAllocator* const  allocator,
 #endif
 
   // Open source file and get its status
-  const int   src_fd = zix_system_open_fd(src, O_RDONLY, 0);
+  const int   src_fd = zix_system_open(src, O_RDONLY, 0);
   struct stat src_stat;
   if (src_fd < 0 || fstat(src_fd, &src_stat)) {
     return finish_copy(-1, src_fd, zix_errno_status(errno));
@@ -200,7 +191,7 @@ zix_copy_file(ZixAllocator* const  allocator,
   // Open a new destination file
   const bool  overwrite = (options == ZIX_COPY_OPTION_OVERWRITE_EXISTING);
   const int   dst_flags = O_WRONLY | O_CREAT | (overwrite ? O_TRUNC : O_EXCL);
-  const int   dst_fd    = zix_system_open_fd(dst, dst_flags, 0644);
+  const int   dst_fd    = zix_system_open(dst, dst_flags, 0644);
   struct stat dst_stat;
   if (dst_fd < 0 || fstat(dst_fd, &dst_stat)) {
     return finish_copy(dst_fd, src_fd, zix_errno_status(errno));
@@ -222,20 +213,14 @@ zix_copy_file(ZixAllocator* const  allocator,
 
   errno = 0;
 
-  // Allocate a block for copying
-  const size_t   align      = zix_system_page_size();
-  const uint32_t block_size = zix_get_block_size(&src_stat, &dst_stat);
-  void* const    block      = zix_aligned_alloc(allocator, align, block_size);
+  // Allocate a block and copy file content one block at a time
+  const uint32_t align = zix_system_page_size();
+  const uint32_t size  = zix_system_max_block_size(&src_stat, &dst_stat, align);
+  BlockBuffer    block = zix_system_new_block(allocator, align, size);
+  void* const    data  = block.buffer ? block.buffer : block.fallback;
+  st                   = copy_blocks(src_fd, dst_fd, data, block.size);
 
-  // Fall back to using a small stack buffer if allocation is unavailable
-  char         stack_buf[512];
-  void* const  buffer      = block ? block : stack_buf;
-  const size_t buffer_size = block ? block_size : sizeof(stack_buf);
-
-  // Copy file content one buffer at a time
-  st = copy_blocks(src_fd, dst_fd, buffer, buffer_size);
-
-  zix_aligned_free(NULL, block);
+  zix_system_free_block(allocator, block);
   return finish_copy(dst_fd, src_fd, st);
 }
 
@@ -387,7 +372,7 @@ stat_file_type(const struct stat* sb)
     ZixFileType type;
   } Mapping;
 
-  static const Mapping map[] = {
+  ZIX_CONSTEXPR Mapping map[] = {
     {S_IFREG, ZIX_FILE_TYPE_REGULAR},
     {S_IFDIR, ZIX_FILE_TYPE_DIRECTORY},
     {S_IFLNK, ZIX_FILE_TYPE_SYMLINK},
