@@ -1,9 +1,10 @@
-// Copyright 2007-2024 David Robillard <d@drobilla.net>
+// Copyright 2007-2025 David Robillard <d@drobilla.net>
 // SPDX-License-Identifier: ISC
 
 #include <zix/filesystem.h>
 
 #include "../errno_status.h"
+#include "../qualifiers.h"
 #include "../zix_config.h"
 #include "win32_util.h"
 
@@ -35,7 +36,7 @@ path_result(ZixAllocator* const allocator, wchar_t* const path)
     return NULL;
   }
 
-  static const wchar_t* const long_prefix = L"\\\\?\\";
+  ZIX_CONSTEXPR wchar_t* const long_prefix = L"\\\\?\\";
 
   const size_t p      = !wcsncmp(path, long_prefix, 4U) ? 4U : 0U;
   char* const  result = zix_wchar_to_utf8(allocator, path + p);
@@ -94,8 +95,6 @@ zix_copy_file(ZixAllocator* const  allocator,
               const char* const    dst,
               const ZixCopyOptions options)
 {
-  (void)allocator;
-
   ArgPathChar* const wsrc = arg_path_new(allocator, src);
   ArgPathChar* const wdst = arg_path_new(allocator, dst);
 
@@ -111,8 +110,8 @@ zix_copy_file(ZixAllocator* const  allocator,
 static inline uint32_t
 lcg32(const uint32_t i)
 {
-  static const uint32_t a = 134775813U;
-  static const uint32_t c = 1U;
+  ZIX_CONSTEXPR uint32_t a = 134775813U;
+  ZIX_CONSTEXPR uint32_t c = 1U;
 
   return (a * i) + c;
 }
@@ -121,8 +120,8 @@ char*
 zix_create_temporary_directory(ZixAllocator* const allocator,
                                const char* const   path_pattern)
 {
-  static const char   chars[] = "ABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789";
-  static const size_t n_chars = sizeof(chars) - 1;
+  ZIX_CONSTEXPR char   chars[] = "ABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789";
+  ZIX_CONSTEXPR size_t n_chars = sizeof(chars) - 1;
 
   // Ensure that the pattern ends with "XXXXXX"
   const size_t length = strlen(path_pattern);
@@ -175,8 +174,8 @@ zix_dir_for_each(const char* const          path,
                  void* const                data,
                  const ZixDirEntryVisitFunc f)
 {
-  static const TCHAR* const dot    = TEXT(".");
-  static const TCHAR* const dotdot = TEXT("..");
+  ZIX_CONSTEXPR TCHAR* const dot    = TEXT(".");
+  ZIX_CONSTEXPR TCHAR* const dotdot = TEXT("..");
 
 #ifdef UNICODE
   const int path_size = MultiByteToWideChar(CP_UTF8, 0, path, -1, NULL, 0);
@@ -228,15 +227,15 @@ zix_file_lock(FILE* const file, const ZixFileLockMode mode)
   return ZIX_STATUS_NOT_SUPPORTED;
 #else
 
-  HANDLE     handle     = (HANDLE)_get_osfhandle(fileno(file));
-  OVERLAPPED overlapped = {0};
+  const intptr_t handle     = _get_osfhandle(fileno(file));
+  OVERLAPPED     overlapped = {0};
 
   const DWORD flags =
     (LOCKFILE_EXCLUSIVE_LOCK |
      (mode == ZIX_FILE_LOCK_TRY ? LOCKFILE_FAIL_IMMEDIATELY : 0));
 
   return zix_windows_status(
-    LockFileEx(handle, flags, 0, UINT32_MAX, UINT32_MAX, &overlapped));
+    LockFileEx((HANDLE)handle, flags, 0, UINT32_MAX, UINT32_MAX, &overlapped));
 #endif
 }
 
@@ -249,11 +248,11 @@ zix_file_unlock(FILE* const file, const ZixFileLockMode mode)
   return ZIX_STATUS_NOT_SUPPORTED;
 #else
 
-  HANDLE     handle     = (HANDLE)_get_osfhandle(fileno(file));
-  OVERLAPPED overlapped = {0};
+  const intptr_t handle     = _get_osfhandle(fileno(file));
+  OVERLAPPED     overlapped = {0};
 
   return zix_windows_status(
-    UnlockFileEx(handle, 0, UINT32_MAX, UINT32_MAX, &overlapped));
+    UnlockFileEx((HANDLE)handle, 0, UINT32_MAX, UINT32_MAX, &overlapped));
 #endif
 }
 
@@ -423,7 +422,7 @@ ZixStatus
 zix_create_symlink(const char* const target_path, const char* const link_path)
 {
 #if USE_CREATESYMBOLICLINK
-  static const DWORD flags = SYMBOLIC_LINK_FLAG_ALLOW_UNPRIVILEGED_CREATE;
+  ZIX_CONSTEXPR DWORD flags = SYMBOLIC_LINK_FLAG_ALLOW_UNPRIVILEGED_CREATE;
 
   ArgPathChar* const wtarget = arg_path_new(NULL, target_path);
   ArgPathChar* const wlink   = arg_path_new(NULL, link_path);
@@ -446,7 +445,7 @@ zix_create_directory_symlink(const char* const target_path,
                              const char* const link_path)
 {
 #if USE_CREATESYMBOLICLINK
-  static const DWORD flags =
+  ZIX_CONSTEXPR DWORD flags =
     SYMBOLIC_LINK_FLAG_DIRECTORY | SYMBOLIC_LINK_FLAG_ALLOW_UNPRIVILEGED_CREATE;
 
   ArgPathChar* const wtarget = arg_path_new(NULL, target_path);
