@@ -16,9 +16,10 @@
 #include <cassert>
 #include <cstdlib>
 #include <filesystem>
-#include <sstream>
 #include <string>
 
+// IWYU pragma: no_include <version>
+
 namespace {
 
 struct BinaryCase {
