@@ -1,8 +1,11 @@
-// Copyright 2020-2022 David Robillard <d@drobilla.net>
+// Copyright 2020-2025 David Robillard <d@drobilla.net>
 // SPDX-License-Identifier: ISC
 
 #undef NDEBUG
 
+#include "failing_allocator.h"
+
+#include <zix/allocator.h>
 #include <zix/path.h>
 #include <zix/string_view.h>
 
@@ -11,7 +14,7 @@
 #include <stdlib.h>
 #include <string.h>
 
-/// Abort if `string` doesn't equal `expected`
+/// Return whether `string` equals `expected`
 static bool
 equal(char* const string, const char* const expected)
 {
@@ -22,7 +25,7 @@ equal(char* const string, const char* const expected)
   return result;
 }
 
-/// Abort if `string` doesn't equal `expected` with preferred separators
+/// Return whether `string` matches `expected` with preferred separators
 static bool
 match(char* const string, const char* const expected)
 {
@@ -37,7 +40,7 @@ match(char* const string, const char* const expected)
   return result;
 }
 
-/// Abort if `view` doesn't equal `expected`
+/// Return whether `view` equals `expected`
 static bool
 view_equal(const ZixStringView view, const char* const expected)
 {
@@ -338,105 +341,96 @@ test_path_lexically_normal(void)
 #endif
 }
 
+static char*
+lexically_relative(const char* const path, const char* const base)
+{
+  return zix_path_lexically_relative(NULL, path, base);
+}
+
 static void
 test_path_lexically_relative(void)
 {
-  assert(equal(zix_path_lexically_relative(NULL, "", ""), "."));
-  assert(equal(zix_path_lexically_relative(NULL, "", "."), "."));
-  assert(equal(zix_path_lexically_relative(NULL, ".", ""), "."));
-  assert(equal(zix_path_lexically_relative(NULL, ".", "."), "."));
-  assert(equal(zix_path_lexically_relative(NULL, "//base", "a"), NULL));
-  assert(equal(zix_path_lexically_relative(NULL, "//host", "//host"), "."));
-  assert(equal(zix_path_lexically_relative(NULL, "//host", "a"), NULL));
-  assert(equal(zix_path_lexically_relative(NULL, "//host/", "//host/"), "."));
-  assert(equal(zix_path_lexically_relative(NULL, "//host/", "a"), NULL));
-  assert(
-    equal(zix_path_lexically_relative(NULL, "//host/a/b", "//host/a/b"), "."));
-  assert(equal(zix_path_lexically_relative(NULL, "/a/b", "/a/"), "b"));
-  assert(equal(zix_path_lexically_relative(NULL, "C:/a/b", "C:/a/"), "b"));
-  assert(equal(zix_path_lexically_relative(NULL, "a", "/"), NULL));
-  assert(equal(zix_path_lexically_relative(NULL, "a", "//host"), NULL));
-  assert(equal(zix_path_lexically_relative(NULL, "a", "//host/"), NULL));
-  assert(equal(zix_path_lexically_relative(NULL, "a", "a"), "."));
-  assert(equal(zix_path_lexically_relative(NULL, "a/b", "/a/b"), NULL));
-  assert(equal(zix_path_lexically_relative(NULL, "a/b", "a/b"), "."));
-  assert(equal(zix_path_lexically_relative(NULL, "a/b/c", "a"), "b/c"));
-  assert(equal(zix_path_lexically_relative(NULL, "a/b/c", "a/b/c"), "."));
-  assert(equal(zix_path_lexically_relative(NULL, "a/b/c/", "a/b/c/"), "."));
-  assert(match(zix_path_lexically_relative(NULL, "../", "../"), "."));
-  assert(match(zix_path_lexically_relative(NULL, "../", "./"), "../"));
-  assert(match(zix_path_lexically_relative(NULL, "../", "a"), "../../"));
-  assert(
-    match(zix_path_lexically_relative(NULL, "../../a", "../b"), "../../a"));
-  assert(match(zix_path_lexically_relative(NULL, "../a", "../b"), "../a"));
-  assert(match(zix_path_lexically_relative(NULL, "/a", "/b/c/"), "../../a"));
-  assert(match(zix_path_lexically_relative(NULL, "/a/b/c", "/a/b/d/"), "../c"));
-  assert(
-    match(zix_path_lexically_relative(NULL, "/a/b/c", "/a/b/d/e/"), "../../c"));
-  assert(match(zix_path_lexically_relative(NULL, "/a/b/c", "/a/d"), "../b/c"));
-  assert(match(zix_path_lexically_relative(NULL, "/a/d", "/a/b/c"), "../../d"));
-  assert(
-    match(zix_path_lexically_relative(NULL, "C:/D/", "C:/D/F.txt"), "../"));
-  assert(match(zix_path_lexically_relative(NULL, "C:/D/F", "C:/D/S/"), "../F"));
-  assert(match(zix_path_lexically_relative(NULL, "C:/D/F", "C:/G"), "../D/F"));
-  assert(
-    match(zix_path_lexically_relative(NULL, "C:/D/F.txt", "C:/D/"), "F.txt"));
-  assert(match(zix_path_lexically_relative(NULL, "C:/E", "C:/D/G"), "../../E"));
-  assert(
-    match(zix_path_lexically_relative(NULL, "C:/a", "C:/b/c/"), "../../a"));
-  assert(
-    match(zix_path_lexically_relative(NULL, "C:/a/b/c", "C:/a/b/d/"), "../c"));
-  assert(match(zix_path_lexically_relative(NULL, "a/b", "c/d"), "../../a/b"));
-  assert(match(zix_path_lexically_relative(NULL, "a/b/c", "../"), NULL));
-  assert(match(zix_path_lexically_relative(NULL, "a/b/c", "../../"), NULL));
-  assert(
-    match(zix_path_lexically_relative(NULL, "a/b/c", "a/b/c/x/y"), "../.."));
+  assert(equal(lexically_relative("", ""), "."));
+  assert(equal(lexically_relative("", "."), "."));
+  assert(equal(lexically_relative(".", ""), "."));
+  assert(equal(lexically_relative(".", "."), "."));
+  assert(equal(lexically_relative("//base", "a"), NULL));
+  assert(equal(lexically_relative("//host", "//host"), "."));
+  assert(equal(lexically_relative("//host", "a"), NULL));
+  assert(equal(lexically_relative("//host/", "//host/"), "."));
+  assert(equal(lexically_relative("//host/", "a"), NULL));
+  assert(equal(lexically_relative("//host/a/b", "//host/a/b"), "."));
+  assert(equal(lexically_relative("/a/b", "/a/"), "b"));
+  assert(equal(lexically_relative("C:/a/b", "C:/a/"), "b"));
+  assert(equal(lexically_relative("a", "/"), NULL));
+  assert(equal(lexically_relative("a", "//host"), NULL));
+  assert(equal(lexically_relative("a", "//host/"), NULL));
+  assert(equal(lexically_relative("a", "a"), "."));
+  assert(equal(lexically_relative("a/b", "/a/b"), NULL));
+  assert(equal(lexically_relative("a/b", "a/b"), "."));
+  assert(equal(lexically_relative("a/b/c", "a"), "b/c"));
+  assert(equal(lexically_relative("a/b/c", "a/b/c"), "."));
+  assert(equal(lexically_relative("a/b/c/", "a/b/c/"), "."));
+  assert(match(lexically_relative("../", "../"), "."));
+  assert(match(lexically_relative("../", "./"), "../"));
+  assert(match(lexically_relative("../", "a"), "../../"));
+  assert(match(lexically_relative("../../a", "../b"), "../../a"));
+  assert(match(lexically_relative("../a", "../b"), "../a"));
+  assert(match(lexically_relative("/a", "/b/c/"), "../../a"));
+  assert(match(lexically_relative("/a/b/c", "/a/b/d/"), "../c"));
+  assert(match(lexically_relative("/a/b/c", "/a/b/d/e/"), "../../c"));
+  assert(match(lexically_relative("/a/b/c", "/a/d"), "../b/c"));
+  assert(match(lexically_relative("/a/d", "/a/b/c"), "../../d"));
+  assert(match(lexically_relative("C:/D/", "C:/D/F.txt"), "../"));
+  assert(match(lexically_relative("C:/D/F", "C:/D/S/"), "../F"));
+  assert(match(lexically_relative("C:/D/F", "C:/G"), "../D/F"));
+  assert(match(lexically_relative("C:/D/F.txt", "C:/D/"), "F.txt"));
+  assert(match(lexically_relative("C:/E", "C:/D/G"), "../../E"));
+  assert(match(lexically_relative("C:/a", "C:/b/c/"), "../../a"));
+  assert(match(lexically_relative("C:/a/b/c", "C:/a/b/d/"), "../c"));
+  assert(match(lexically_relative("a/b", "c/d"), "../../a/b"));
+  assert(match(lexically_relative("a/b/c", "../"), NULL));
+  assert(match(lexically_relative("a/b/c", "../../"), NULL));
+  assert(match(lexically_relative("a/b/c", "a/b/c/x/y"), "../.."));
 
 #ifdef _WIN32
-  assert(equal(zix_path_lexically_relative(NULL, "/", "a"), "/"));
-  assert(equal(zix_path_lexically_relative(NULL, "//host", "//host/"), NULL));
-  assert(equal(zix_path_lexically_relative(NULL, "//host/", "//host"), "/"));
-  assert(equal(zix_path_lexically_relative(NULL, "//host/", "/a"), NULL));
-  assert(equal(zix_path_lexically_relative(NULL, "/a", "//host"), NULL));
-  assert(equal(zix_path_lexically_relative(NULL, "/a", "//host/"), NULL));
-  assert(equal(zix_path_lexically_relative(NULL, "C:/D/", "C:F.txt"), NULL));
-  assert(equal(zix_path_lexically_relative(NULL, "C:/D/S/", "F.txt"), NULL));
-  assert(equal(zix_path_lexically_relative(NULL, "C:F", "C:/D/"), NULL));
-  assert(equal(zix_path_lexically_relative(NULL, "C:F", "D:G"), NULL));
-  assert(equal(zix_path_lexically_relative(NULL, "F", "C:/D/S/"), NULL));
-  assert(match(zix_path_lexically_relative(NULL, "C:../", "C:../"), "."));
-  assert(match(zix_path_lexically_relative(NULL, "C:../", "C:./"), "../"));
-  assert(match(zix_path_lexically_relative(NULL, "C:../", "C:a"), "../../"));
-  assert(
-    match(zix_path_lexically_relative(NULL, "C:../../a", "C:../b"), "../../a"));
-  assert(match(zix_path_lexically_relative(NULL, "C:../a", "C:../b"), "../a"));
-  assert(match(zix_path_lexically_relative(NULL, "C:a/b/c", "C:../"), NULL));
-  assert(match(zix_path_lexically_relative(NULL, "C:a/b/c", "C:../../"), NULL));
+  assert(equal(lexically_relative("/", "a"), "/"));
+  assert(equal(lexically_relative("//host", "//host/"), NULL));
+  assert(equal(lexically_relative("//host/", "//host"), "/"));
+  assert(equal(lexically_relative("//host/", "/a"), NULL));
+  assert(equal(lexically_relative("/a", "//host"), NULL));
+  assert(equal(lexically_relative("/a", "//host/"), NULL));
+  assert(equal(lexically_relative("C:/D/", "C:F.txt"), NULL));
+  assert(equal(lexically_relative("C:/D/S/", "F.txt"), NULL));
+  assert(equal(lexically_relative("C:F", "C:/D/"), NULL));
+  assert(equal(lexically_relative("C:F", "D:G"), NULL));
+  assert(equal(lexically_relative("F", "C:/D/S/"), NULL));
+  assert(match(lexically_relative("C:../", "C:../"), "."));
+  assert(match(lexically_relative("C:../", "C:./"), "../"));
+  assert(match(lexically_relative("C:../", "C:a"), "../../"));
+  assert(match(lexically_relative("C:../../a", "C:../b"), "../../a"));
+  assert(match(lexically_relative("C:../a", "C:../b"), "../a"));
+  assert(match(lexically_relative("C:a/b/c", "C:../"), NULL));
+  assert(match(lexically_relative("C:a/b/c", "C:../../"), NULL));
 #else
-  assert(equal(zix_path_lexically_relative(NULL, "/", "a"), NULL));
-  assert(equal(zix_path_lexically_relative(NULL, "//host", "//host/"), "."));
-  assert(equal(zix_path_lexically_relative(NULL, "//host/", "//host"), "."));
-  assert(equal(zix_path_lexically_relative(NULL, "//host/", "/a"), "../host/"));
-  assert(equal(zix_path_lexically_relative(NULL, "/a", "//host"), "../a"));
-  assert(equal(zix_path_lexically_relative(NULL, "/a", "//host/"), "../a"));
-  assert(
-    equal(zix_path_lexically_relative(NULL, "C:/D/", "C:F.txt"), "../C:/D/"));
-  assert(
-    equal(zix_path_lexically_relative(NULL, "C:/D/S/", "F.txt"), "../C:/D/S/"));
-  assert(equal(zix_path_lexically_relative(NULL, "C:F", "C:/D/"), "../../C:F"));
-  assert(equal(zix_path_lexically_relative(NULL, "C:F", "D:G"), "../C:F"));
-  assert(
-    equal(zix_path_lexically_relative(NULL, "F", "C:/D/S/"), "../../../F"));
-  assert(match(zix_path_lexically_relative(NULL, "C:../", "C:../"), "."));
-  assert(match(zix_path_lexically_relative(NULL, "C:../", "C:./"), "../C:../"));
-  assert(match(zix_path_lexically_relative(NULL, "C:../", "C:a"), "../C:../"));
-  assert(
-    match(zix_path_lexically_relative(NULL, "C:../../a", "C:../b"), "../../a"));
-  assert(match(zix_path_lexically_relative(NULL, "C:../a", "C:../b"), "../a"));
-  assert(
-    match(zix_path_lexically_relative(NULL, "C:a/b/c", "C:../"), "../C:a/b/c"));
-  assert(
-    match(zix_path_lexically_relative(NULL, "C:a/b/c", "C:../../"), "C:a/b/c"));
+  assert(equal(lexically_relative("/", "a"), NULL));
+  assert(equal(lexically_relative("//host", "//host/"), "."));
+  assert(equal(lexically_relative("//host/", "//host"), "."));
+  assert(equal(lexically_relative("//host/", "/a"), "../host/"));
+  assert(equal(lexically_relative("/a", "//host"), "../a"));
+  assert(equal(lexically_relative("/a", "//host/"), "../a"));
+  assert(equal(lexically_relative("C:/D/", "C:F.txt"), "../C:/D/"));
+  assert(equal(lexically_relative("C:/D/S/", "F.txt"), "../C:/D/S/"));
+  assert(equal(lexically_relative("C:F", "C:/D/"), "../../C:F"));
+  assert(equal(lexically_relative("C:F", "D:G"), "../C:F"));
+  assert(equal(lexically_relative("F", "C:/D/S/"), "../../../F"));
+  assert(match(lexically_relative("C:../", "C:../"), "."));
+  assert(match(lexically_relative("C:../", "C:./"), "../C:../"));
+  assert(match(lexically_relative("C:../", "C:a"), "../C:../"));
+  assert(match(lexically_relative("C:../../a", "C:../b"), "../../a"));
+  assert(match(lexically_relative("C:../a", "C:../b"), "../a"));
+  assert(match(lexically_relative("C:a/b/c", "C:../"), "../C:a/b/c"));
+  assert(match(lexically_relative("C:a/b/c", "C:../../"), "C:a/b/c"));
 #endif
 }
 
@@ -924,10 +918,107 @@ test_path_is_absolute(void)
   assert(zix_path_is_relative("C:\\a\\b"));
   assert(zix_path_is_relative("D:/a/b"));
   assert(zix_path_is_relative("D:\\a\\b"));
-
 #endif
 }
 
+static void
+test_null_queries(void)
+{
+  assert(!zix_path_has_root_path(NULL));
+  assert(!zix_path_has_root_name(NULL));
+  assert(!zix_path_has_root_directory(NULL));
+  assert(!zix_path_has_relative_path(NULL));
+  assert(!zix_path_has_parent_path(NULL));
+  assert(!zix_path_has_filename(NULL));
+  assert(!zix_path_has_stem(NULL));
+  assert(!zix_path_has_extension(NULL));
+  assert(!zix_path_is_absolute(NULL));
+  assert(zix_path_is_relative(NULL));
+}
+
+typedef char* (*AllocCheckFunc)(ZixAllocator*);
+
+static char*
+check_join_alloc(ZixAllocator* const allocator)
+{
+  return zix_path_join(allocator, "a", "b");
+}
+
+static char*
+check_join_empty_alloc(ZixAllocator* const allocator)
+{
+  return zix_path_join(allocator, "", "b");
+}
+
+static char*
+check_preferred_alloc(ZixAllocator* const allocator)
+{
+  return zix_path_preferred(allocator, "/a/b");
+}
+
+static char*
+check_lexically_normal_alloc(ZixAllocator* const allocator)
+{
+  return zix_path_lexically_normal(allocator, "./a/././b/./");
+}
+
+static char*
+check_lexically_normal_empty_alloc(ZixAllocator* const allocator)
+{
+  return zix_path_lexically_normal(allocator, "");
+}
+
+static char*
+check_lexically_relative_alloc(ZixAllocator* const allocator)
+{
+  return zix_path_lexically_relative(allocator, "../../a", "../b");
+}
+
+static char*
+check_lexically_relative_equal_alloc(ZixAllocator* const allocator)
+{
+  return zix_path_lexically_relative(allocator, "a", "a");
+}
+
+static char*
+check_lexically_relative_dot_alloc(ZixAllocator* const allocator)
+{
+  return zix_path_lexically_relative(allocator, "", ".");
+}
+
+static void
+check_failed_alloc(const AllocCheckFunc check)
+{
+  ZixFailingAllocator allocator = zix_failing_allocator();
+
+  // Successfully perform check to count the number of allocations
+  {
+    char* const result = check(&allocator.base);
+    assert(result);
+    zix_free(&allocator.base, result);
+  }
+
+  // Test that each allocation failing is handled gracefully
+  const size_t n_new_allocs = zix_failing_allocator_reset(&allocator, 0);
+  for (size_t i = 0U; i < n_new_allocs; ++i) {
+    zix_failing_allocator_reset(&allocator, i);
+    assert(!check(&allocator.base));
+  }
+}
+
+static void
+test_failed_alloc(void)
+{
+  check_failed_alloc(check_join_alloc);
+  check_failed_alloc(check_join_empty_alloc);
+  check_failed_alloc(check_preferred_alloc);
+  check_failed_alloc(check_lexically_normal_alloc);
+  check_failed_alloc(check_lexically_normal_empty_alloc);
+  check_failed_alloc(check_lexically_relative_alloc);
+  check_failed_alloc(check_lexically_relative_equal_alloc);
+  check_failed_alloc(check_lexically_relative_dot_alloc);
+}
+
 int
 main(void)
 {
@@ -938,11 +1029,14 @@ main(void)
   test_path_stem();
   test_path_extension();
   test_path_is_absolute();
+  test_null_queries();
 
   test_path_join();
   test_path_preferred();
   test_path_lexically_normal();
   test_path_lexically_relative();
 
+  test_failed_alloc();
+
   return 0;
 }
